@@ -3,6 +3,7 @@
                              -------------------
     begin                : Fri Sep 8 2000
     copyright            : (C) 2000 by Andrea Rizzi
+			   (C) 2005 by Stanislav Visnovsky
     email                : rizzi@kde.org
  ***************************************************************************/
 
@@ -14,7 +15,17 @@
   Andrea Rizzi rizzi@kde.org
 
   License GPL v 2.0
-
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of this program with any edition of       *
+ *   the Qt library by Trolltech AS, Norway (or with modified versions     *
+ *   of Qt that use the same license as Qt), and distribute linked         *
+ *   combinations including the two.  You must obey the GNU General        *
+ *   Public License in all respects for all of the code used other than    *
+ *   Qt. If you modify this file, you may extend this exception to         *
+ *   your version of the file, but you are not obligated to do so.  If     *
+ *   you do not wish to do so, delete this exception statement from        *
+ *   your version.                                                         *
 */
 
 #include <qtextedit.h>
@@ -113,7 +124,7 @@ KDBSearchEngine::loadDatabase (QString database, bool noask = false)
 					     i18n
 					     ("Database folder does not exist:\n%1\n"
 					      "Do you want to create it now?").
-					     arg (database)) ==
+					     arg (database), QString::null, i18n("Create Folder"), i18n("Do Not Create")) ==
 	      KMessageBox::Yes)
 	    {
 		asked = true;
@@ -186,24 +197,24 @@ KDBSearchEngine::loadDatabase (QString database, bool noask = false)
 
 		      // rename the old files
 		      KIO::NetAccess::copy (KURL (transFile + ",old"),
-					    KURL (transFile));
+					    KURL (transFile), 0);
 		      KIO::NetAccess::
 			  copy (KURL
 				(database + "/wordsindex." + lang +
 				 ".db,old"),
 				KURL (database + "/wordsindex." + lang +
-				      ".db"));
+				      ".db"), 0);
 		      KIO::NetAccess::
 			  copy (KURL
 				(database + "/keysindex." + lang + ".db,old"),
 				KURL (database + "/keysindex." + lang +
-				      ".db"));
+				      ".db"), 0);
 		      KIO::NetAccess::
 			  copy (KURL
 				(database + "/catalogsinfo." + lang +
 				 ".db,old"),
 				KURL (database + "/catalogsinfo." + lang +
-				      ".db"));
+				      ".db"), 0);
 
 		      QFile::remove (transFile + ",old");
 		      QFile::remove (database + "/wordsindex." + lang +
@@ -218,17 +229,17 @@ KDBSearchEngine::loadDatabase (QString database, bool noask = false)
 	    {
 		// rename the old files
 		KIO::NetAccess::copy (KURL (transFile + ",old"),
-				      KURL (transFile));
+				      KURL (transFile), 0);
 		KIO::NetAccess::
 		    copy (KURL (database + "/wordsindex." + lang + ".db,old"),
-			  KURL (database + "/wordsindex." + lang + ".db"));
+			  KURL (database + "/wordsindex." + lang + ".db"), 0);
 		KIO::NetAccess::
 		    copy (KURL (database + "/keysindex." + lang + ".db,old"),
-			  KURL (database + "/keysindex." + lang + ".db"));
+			  KURL (database + "/keysindex." + lang + ".db"), 0);
 		KIO::NetAccess::
 		    copy (KURL
 			  (database + "/catalogsinfo." + lang + ".db,old"),
-			  KURL (database + "/catalogsinfo." + lang + ".db"));
+			  KURL (database + "/catalogsinfo." + lang + ".db"), 0);
 
 		QFile::remove (transFile + ",old");
 		QFile::remove (database + "/wordsindex." + lang + ".db,old");
@@ -244,7 +255,7 @@ KDBSearchEngine::loadDatabase (QString database, bool noask = false)
 		if (asked
 		    || KMessageBox::questionYesNo (0,
 						   i18n
-						   ("Database files not found.\nDo you want to create them now?"))
+						   ("Database files not found.\nDo you want to create them now?"), QString::null, i18n("Create"), i18n("Do Not Create"))
 		    == KMessageBox::Yes)
 		  {
 		      //fprintf(stderr,"SI\n");
@@ -726,31 +737,53 @@ KDBSearchEngine::startSearchNow (int searchmode)
     if (gk)
       {
 	  goodkeys = searchWords (mainRequest, thre, threorig, listmax);	//FIX IT, mainReq?
+	  if (stopNow)
+	  {
+		stopNow = false;
+		searching = false;
+		emit finished ();
+		return false;
+	  }
 	  if (goodkeys.count () == 0)
 	      gk = false;	// if no good keys, use the whole database
       }
+      
+    // prepare progress values
     totalprogress = gk ? goodkeys.count () : totalRecord;
     int step = (totalprogress / 30) + 1;
+    if( step > 100 )
+	step = 100;
 
+    emit progress (0);
+    kapp->processEvents (100);
+    if (stopNow)
+    {
+	stopNow = false;
+	searching = false;
+	emit finished ();
+	return true;	// No error, stopped!
+    }
+    
+    
     for (item = gk ? (dm->getItem (goodkeys[0])) : (dm->firstItem ());
 	 !item.isNull ();
 	 item = gk ? (dm->getItem (goodkeys[++k])) : (dm->nextItem ()))
       {
 
-
 //   Emit progress, process event and check stop now
 	  if (count % step == 0)
 	    {
 		emit progress (100 * count / /*QMAX( */
 			       totalprogress /*,1) */ );
 		kapp->processEvents (100);
-	    }
-	  if (stopNow)
-	    {
-		stopNow = false;
-		searching = false;
-		emit finished ();
-		return true;	// No error, stopped!
+
+		if (stopNow)
+		{
+	    	    stopNow = false;
+		    searching = false;
+		    emit finished ();
+		    return true;	// No error, stopped!
+		}
 	    }
 
 	  //  fprintf(stderr,"%s\n",(const char *)item.key.utf8());
@@ -788,8 +821,7 @@ KDBSearchEngine::startSearchNow (int searchmode)
 	  it = searchList.begin ();
 	  idMod = &((*it).string);
 	  bool foundExact = false;
-
-
+	  
 	  for (it1 = searchStringList.begin ();
 	       it1 != searchStringList.end (); it1++)
 	    {
@@ -858,9 +890,6 @@ KDBSearchEngine::startSearchNow (int searchmode)
 		if (equal || contains || contained || regexp || allkey)
 		  {
 
-//           kdDebug(0) << QString("Found") << endl;
-
-
 		      if (equal)
 			  foundExact = true;
 
@@ -1688,6 +1717,10 @@ QValueList < KeyAndScore > KDBSearchEngine::searchWords (QString phrase,
 				      progress (100 * count / totalprogress);
 				      kapp->processEvents (100);
 				  }
+				  if (stopNow)
+				  {
+					return keylist;
+				  }
 
 				currentloc[j]++;
 				if (numofloc[j] == currentloc[j])	//End reached
