@@ -41,7 +41,6 @@
 #include <kinstance.h>
 #include <kio/netaccess.h>
 
-#include <qregexp.h>
 #include <qtextstream.h>
 #include <qtimer.h>
 
@@ -747,21 +746,21 @@ void TmxCompendium::recheckData()
     loading=false;
 }
 
-QString TmxCompendium::maskString(QString s)
+QString TmxCompendium::maskString(QString s) const
 {
-    s.replace(QRegExp("\\\\"),"\\\\");
-    s.replace(QRegExp("\\?"),"\\?");
-    s.replace(QRegExp("\\["),"\\[");
-    s.replace(QRegExp("\\."),"\\.");
-    s.replace(QRegExp("\\*"),"\\*");
-    s.replace(QRegExp("\\+"),"\\+");
-    s.replace(QRegExp("\\^"),"\\^");
-    s.replace(QRegExp("\\$"),"\\$");
-    s.replace(QRegExp("\\("),"\\(");
-    s.replace(QRegExp("\\)"),"\\)");
-    s.replace(QRegExp("\\{"),"\\{");
-    s.replace(QRegExp("\\}"),"\\}");
-    s.replace(QRegExp("\\|"),"\\|");
+    s.replace('\\',"\\\\");
+    s.replace('?',"\\?");
+    s.replace('[',"\\[");
+    s.replace('.',"\\.");
+    s.replace('*',"\\*");
+    s.replace('+',"\\+");
+    s.replace('^',"\\^");
+    s.replace('$',"\\$");
+    s.replace('(',"\\(");
+    s.replace(')',"\\)");
+    s.replace('{',"\\{");
+    s.replace('}',"\\}");
+    s.replace('|',"\\|");
     
     return s;
 }
