@@ -1,9 +1,9 @@
 /***************************************************************************
                           sqlwriter.h  -  description
                              -------------------
-    begin                : 10.02.2003   
+    begin                : 10.02.2003
     copyright            : (C) 2003 by Nikolaus Gradwohl
-    email                : guru@local-guru.net 
+    email                : guru@local-guru.net
  ***************************************************************************/
 
 /***************************************************************************
@@ -31,7 +31,7 @@ class SQLWriter : public SimpleCodeGenerator {
     Q_OBJECT
 public:
 
-    SQLWriter( UMLDoc * parent, const char* name = 0 );
+    SQLWriter();
     virtual ~SQLWriter();
 
     /**
@@ -40,17 +40,15 @@ public:
      */
     virtual void writeClass(UMLClassifier *c);
 
-    /**
-     * checks whether type is "SQLWriter"
-     *
-     * @param type
-     */
-    virtual bool isType (QString & type);
-
     /**
      * returns "SQL"
      */
-    virtual QString getLanguage();
+    virtual Uml::Programming_Language getLanguage();
+
+    /**
+     * Reimplement method from CodeGenerator.
+     */
+    virtual QStringList defaultDatatypes();
 
     /**
      * get list of reserved keywords
