@@ -1,8 +1,3 @@
- /*
-  *  copyright (C) 2002-2004
-  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
-  */
-
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2002-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef NOTEDIALOG_H
@@ -28,28 +25,31 @@ class QLineEdit;
 class QTextEdit;
 
 class NoteDialog : public KDialogBase {
-public:
-	/**
-	 *	Constructs an NoteDialog.
-	 */
-	NoteDialog( QWidget * parent, NoteWidget * pNote );
 
-	/**
-	 *	Standard deconstructor.
-	 */
-	~NoteDialog();
+   Q_OBJECT
+
+public:
+    /**
+     *  Constructs an NoteDialog.
+     */
+    NoteDialog( QWidget * parent, NoteWidget * pNote );
+
+    /**
+     *  Standard deconstructor.
+     */
+    ~NoteDialog();
 
 public slots:
-	void slotOk();
+    void slotOk();
 private:
-	//GUI widgets
-	QGroupBox * m_pDocGB;
-	QTextEdit * m_pDocTE;
+    //GUI widgets
+    QGroupBox * m_pDocGB;
+    QTextEdit * m_pDocTE;
 
-	/**
-	 *	Note widget to show documentation for.
-	 */
-	NoteWidget * m_pNoteWidget;
+    /**
+     *  Note widget to show documentation for.
+     */
+    NoteWidget * m_pNoteWidget;
 };
 
 #endif
