@@ -5,8 +5,8 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *  copyright (C) 2002-2006                                                *
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>                  *
+ *   copyright (C) 2002-2006                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 // own header
@@ -94,42 +94,35 @@ bool UMLObject::acceptAssociationType(Uml::Association_Type)
 
 void UMLObject::setID(Uml::IDType NewID) {
     m_nId = NewID;
-    emit modified();
+    emitModified();
 }
 
 void UMLObject::setName(const QString &strName) {
     m_Name = strName;
-    emit modified();
+    emitModified();
 }
 
 QString UMLObject::getName() const {
     return m_Name;
 }
 
-QString UMLObject::getFullyQualifiedName(QString separator,
+QString UMLObject::getFullyQualifiedName(const QString& separator,
                                          bool includeRoot /* = false */) const {
     QString fqn;
     if (m_pUMLPackage) {
         bool skipPackage = false;
         if (!includeRoot) {
             UMLDoc *umldoc = UMLApp::app()->getDocument();
-            if (m_pUMLPackage == umldoc->getDatatypeFolder()) {
+            if (umldoc->rootFolderType(m_pUMLPackage) != Uml::N_MODELTYPES ||
+                m_pUMLPackage == umldoc->getDatatypeFolder())
                 skipPackage = true;
-            } else {
-                for (int i = 0; i < Uml::N_MODELTYPES; i++) {
-                    const Uml::Model_Type mt = (Uml::Model_Type)i;
-                    if (m_pUMLPackage == umldoc->getRootFolder(mt)) {
-                        skipPackage = true;
-                        break;
-                    }
-                }
-            }
         }
         if (!skipPackage) {
-            if (separator.isEmpty())
-                separator = UMLApp::app()->activeLanguageScopeSeparator();
-            fqn = m_pUMLPackage->getFullyQualifiedName(separator, includeRoot);
-            fqn.append(separator);
+            QString tempSeparator = separator;
+            if (tempSeparator.isEmpty())
+                tempSeparator = UMLApp::app()->activeLanguageScopeSeparator();
+            fqn = m_pUMLPackage->getFullyQualifiedName(tempSeparator, includeRoot);
+            fqn.append(tempSeparator);
         }
     }
     fqn.append(m_Name);
@@ -209,7 +202,7 @@ void UMLObject::copyInto(UMLObject *rhs) const
 
     // Hope that the parent from QObject is okay.
     if (rhs->parent() != parent())
-        kdDebug() << "copyInto has a wrong parent" << endl;
+        kDebug() << "copyInto has a wrong parent" << endl;
 }
 
 
@@ -219,7 +212,7 @@ bool UMLObject::getAbstract() const{
 
 void UMLObject::setAbstract(bool bAbstract) {
     m_bAbstract = bAbstract;
-    emit modified();
+    emitModified();
 }
 
 void UMLObject::setInPaste(bool bInPaste /* =true */) {
@@ -235,17 +228,19 @@ bool UMLObject::getStatic() const
 void UMLObject::setStatic(bool bStatic)
 {
     m_bStatic = bStatic;
-    emit modified();
+    emitModified();
 }
 
 void UMLObject::emitModified()
 {
-    emit modified();
+    UMLDoc *umldoc = UMLApp::app()->getDocument();
+    if (! umldoc->loading())
+        emit modified();
 }
 
 void UMLObject::setDoc(const QString &d) {
     m_Doc = d;
-    emit modified();
+    //emit modified();  No, this is done centrally at DocWindow::updateDocumentation()
 }
 
 Uml::Object_Type UMLObject::getBaseType() const {
@@ -270,7 +265,7 @@ Uml::Visibility UMLObject::getVisibility() const {
 
 void UMLObject::setVisibility(Uml::Visibility s) {
     m_Vis = s;
-    emit modified();
+    emitModified();
 }
 
 void UMLObject::setUMLStereotype(UMLStereotype *stereo) {
@@ -289,7 +284,7 @@ void UMLObject::setUMLStereotype(UMLStereotype *stereo) {
     }
     m_pStereotype = stereo;
     // TODO: don't emit modified() if predefined folder
-    emit modified();
+    emitModified();
 }
 
 void UMLObject::setStereotype(const QString &_name) {
@@ -308,13 +303,13 @@ void UMLObject::setPackage(const QString &_name) {
         UMLDoc* umldoc = UMLApp::app()->getDocument();
         pkgObj = umldoc->findUMLObject(_name);
         if (pkgObj == NULL) {
-            kdDebug() << "UMLObject::setPackage: creating UMLPackage "
+            kDebug() << "UMLObject::setPackage: creating UMLPackage "
                 << _name << " for " << m_Name << endl;
             pkgObj = Import_Utils::createUMLObject(Uml::ot_Package, _name);
         } else {
             const Uml::Object_Type ot = pkgObj->getBaseType();
             if (ot != Uml::ot_Package && ot != Uml::ot_Folder && ot != Uml::ot_Component) {
-                kdError() << "UMLObject::setPackage(" << m_Name << "): "
+                kError() << "UMLObject::setPackage(" << m_Name << "): "
                     << "existing " << _name << " is not a container" << endl;
                 // This should not happen - if it does, there may be further problems.
                 // A container name should not overlap with another name in the same scope.
@@ -327,7 +322,7 @@ void UMLObject::setPackage(const QString &_name) {
 
 void UMLObject::setUMLPackage(UMLPackage* pPkg) {
     m_pUMLPackage = pPkg;
-    emit modified();
+    emitModified();
 }
 
 const UMLStereotype * UMLObject::getUMLStereotype() {
@@ -343,11 +338,14 @@ QString UMLObject::getStereotype(bool includeAdornments /* = false */) const {
     return name;
 }
 
-QString UMLObject::getPackage(QString separator, bool includeRoot) {
-    QString fqn = getFullyQualifiedName(separator, includeRoot);
-    if (!fqn.contains(separator))
+QString UMLObject::getPackage(const QString& separator, bool includeRoot) {
+    QString tempSeparator = separator;
+    if (tempSeparator.isEmpty())
+        tempSeparator = UMLApp::app()->activeLanguageScopeSeparator();
+    QString fqn = getFullyQualifiedName(tempSeparator, includeRoot);
+    if (!fqn.contains(tempSeparator))
         return "";
-    QString scope = fqn.left(fqn.length() - separator.length() - m_Name.length());
+    QString scope = fqn.left(fqn.length() - tempSeparator.length() - m_Name.length());
     return scope;
 }
 
@@ -371,7 +369,7 @@ QString UMLObject::getSecondaryId() const {
     return m_SecondaryId;
 }
 
-void UMLObject::setSecondaryId(QString id) {
+void UMLObject::setSecondaryId(const QString& id) {
     m_SecondaryId = id;
 }
 
@@ -379,7 +377,7 @@ QString UMLObject::getSecondaryFallback() const {
     return m_SecondaryFallback;
 }
 
-void UMLObject::setSecondaryFallback(QString id) {
+void UMLObject::setSecondaryFallback(const QString& id) {
     m_SecondaryFallback = id;
 }
 
@@ -400,7 +398,7 @@ bool UMLObject::resolveRef() {
         return true;
     }
 #ifdef VERBOSE_DEBUGGING
-    kdDebug() << "UMLObject::resolveRef(" << m_Name << "): m_SecondaryId is "
+    kDebug() << "UMLObject::resolveRef(" << m_Name << "): m_SecondaryId is "
               << m_SecondaryId << endl;
 #endif
     UMLDoc *pDoc = UMLApp::app()->getDocument();
@@ -418,22 +416,27 @@ bool UMLObject::resolveRef() {
             maybeSignalObjectCreated();
             return true;
         }
-    }
-    if (!pDoc->isNativeXMIFile() || !m_SecondaryId.contains(QRegExp("\\D"))) {
         if (m_SecondaryFallback.isEmpty()) {
-            kdError() << "UMLObject::resolveRef(" << m_Name
-                      << "): cannot find type with id "
-                      << m_SecondaryId << endl;
-            return false;
+            kDebug() << "UMLObject::resolveRef: object with xmi.id=" << m_SecondaryId
+                << " not found, setting to undef" << endl;
+            UMLFolder *datatypes = pDoc->getDatatypeFolder();
+            m_pSecondary = Object_Factory::createUMLObject(Uml::ot_Datatype, "undef", datatypes, false);
+            return true;
         }
-#ifdef VERBOSE_DEBUGGING
-        kdDebug() << "UMLObject::resolveRef(" << m_Name 
-                  << "): could not resolve secondary ID " << m_SecondaryId
-                  << ", using secondary fallback " << m_SecondaryFallback
-                  << endl;
-#endif
-        m_SecondaryId = m_SecondaryFallback;
     }
+    if (m_SecondaryFallback.isEmpty()) {
+        kError() << "UMLObject::resolveRef(" << m_Name
+            << "): cannot find type with id "
+            << m_SecondaryId << endl;
+        return false;
+    }
+#ifdef VERBOSE_DEBUGGING
+    kDebug() << "UMLObject::resolveRef(" << m_Name
+              << "): could not resolve secondary ID " << m_SecondaryId
+              << ", using secondary fallback " << m_SecondaryFallback
+              << endl;
+#endif
+    m_SecondaryId = m_SecondaryFallback;
     // Assume we're dealing with the older Umbrello format where
     // the type name was saved in the "type" attribute rather
     // than the xmi.id of the model object of the attribute type.
@@ -443,31 +446,29 @@ bool UMLObject::resolveRef() {
         maybeSignalObjectCreated();
         return true;
     }
-    //pDoc->setIsOldFile(true);
     // Work around Object_Factory::createUMLObject()'s incapability
     // of on-the-fly scope creation:
     if (m_SecondaryId.contains("::")) {
         // TODO: Merge Import_Utils::createUMLObject() into Object_Factory::createUMLObject()
-        m_pSecondary = Import_Utils::createUMLObject(Uml::ot_UMLObject, m_SecondaryId, NULL);
+        m_pSecondary = Import_Utils::createUMLObject(Uml::ot_UMLObject, m_SecondaryId, m_pUMLPackage);
         if (m_pSecondary) {
             if (Import_Utils::newUMLObjectWasCreated()) {
                 maybeSignalObjectCreated();
                 kapp->processEvents();
-                kdDebug() << "UMLObject::resolveRef: Import_Utils::createUMLObject() "
+                kDebug() << "UMLObject::resolveRef: Import_Utils::createUMLObject() "
                           << "created a new type for " << m_SecondaryId << endl;
             } else {
-                kdDebug() << "UMLObject::resolveRef: Import_Utils::createUMLObject() "
+                kDebug() << "UMLObject::resolveRef: Import_Utils::createUMLObject() "
                           << "returned an existing type for " << m_SecondaryId << endl;
             }
             m_SecondaryId = "";
             return true;
         }
-        kdError() << "UMLObject::resolveRef: Import_Utils::createUMLObject() "
+        kError() << "UMLObject::resolveRef: Import_Utils::createUMLObject() "
                   << "failed to create a new type for " << m_SecondaryId << endl;
         return false;
     }
-    kdDebug() << "UMLObject::resolveRef: Creating new type for "
-    << m_SecondaryId << endl;
+    kDebug() << "UMLObject::resolveRef: Creating new type for " << m_SecondaryId << endl;
     // This is very C++ specific - we rely on  some '*' or
     // '&' to decide it's a ref type. Plus, we don't recognize
     // typedefs of ref types.
@@ -542,10 +543,40 @@ bool UMLObject::load( QDomElement& ) {
     return true;
 }
 
+bool UMLObject::loadStereotype(QDomElement & element) {
+    QString tag = element.tagName();
+    if (!Uml::tagEq(tag, "stereotype"))
+        return false;
+    QString stereo = element.attribute("xmi.value", "");
+    if (stereo.isEmpty() && element.hasChildNodes()) {
+        /* like so:
+         <UML:ModelElement.stereotype>
+           <UML:Stereotype xmi.idref = '07CD'/>
+         </UML:ModelElement.stereotype>
+         */
+        QDomNode stereoNode = element.firstChild();
+        QDomElement stereoElem = stereoNode.toElement();
+        tag = stereoElem.tagName();
+        if (Uml::tagEq(tag, "Stereotype")) {
+            stereo = stereoElem.attribute("xmi.idref", "");
+        }
+    }
+    if (stereo.isEmpty())
+        return false;
+    Uml::IDType stereoID = STR2ID(stereo);
+    UMLDoc *pDoc = UMLApp::app()->getDocument();
+    m_pStereotype = pDoc->findStereotypeById(stereoID);
+    if (m_pStereotype)
+        m_pStereotype->incrRefCount();
+    else
+        m_SecondaryId = stereo;  // leave it to resolveRef()
+    return true;
+}
+
 bool UMLObject::loadFromXMI( QDomElement & element) {
     UMLDoc* umldoc = UMLApp::app()->getDocument();
     if (umldoc == NULL) {
-        kdError() << "UMLObject::loadFromXMI: umldoc is NULL" << endl;
+        kError() << "UMLObject::loadFromXMI: umldoc is NULL" << endl;
         return false;
     }
     // Read the name first so that if we encounter a problem, the error
@@ -558,12 +589,24 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
             // of UMLRole objects.
             m_nId = UniqueID::gen();
         } else {
-            kdError() << "UMLObject::loadFromXMI(" << m_Name
+            kError() << "UMLObject::loadFromXMI(" << m_Name
             << "): nonexistent or illegal xmi.id" << endl;
             return false;
         }
     } else {
         m_nId = STR2ID(id);
+        if (m_BaseType == Uml::ot_Role) {
+            // Some older Umbrello versions had a problem with xmi.id's
+            // of other objects being reused for the UMLRole, see e.g.
+            // attachment 21179 at http://bugs.kde.org/147988 .
+            // If the xmi.id is already being used then we generate a new one.
+            UMLObject *o = umldoc->findObjectById(m_nId);
+            if (o) {
+                kDebug() << "loadFromXMI(UMLRole): id " << id
+                    << " is already in use, generating a new one." << endl;
+                m_nId = UniqueID::gen();
+            }
+        }
     }
 
     if (element.hasAttribute("documentation"))  // for bkwd compat.
@@ -583,7 +626,7 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
             if (nScope >= Uml::Visibility::Public && nScope <= Uml::Visibility::Protected)
               m_Vis = (Uml::Visibility::Value)nScope;
             else
-                kdError() << "UMLObject::loadFromXMI(" << m_Name
+                kError() << "UMLObject::loadFromXMI(" << m_Name
                 << "): illegal scope" << endl;  // soft error
         }
     } else {
@@ -599,19 +642,16 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
     }
 
     QString stereo = element.attribute( "stereotype", "" );
-    if (!stereo.isEmpty() && stereo != "-1") {
-        if (stereo.contains(QRegExp("\\D"))) {
-            // Old versions saved the stereotype name instead of the xmi.id.
-            setStereotype( stereo );
+    if (!stereo.isEmpty()) {
+        Uml::IDType stereoID = STR2ID(stereo);
+        m_pStereotype = umldoc->findStereotypeById(stereoID);
+        if (m_pStereotype) {
+            m_pStereotype->incrRefCount();
         } else {
-            Uml::IDType stereoID = STR2ID(stereo);
-            m_pStereotype = umldoc->findStereotypeById(stereoID);
-            if (m_pStereotype)
-                m_pStereotype->incrRefCount();
-            else
-                kdError() << "UMLObject::loadFromXMI(" << m_Name << "): "
+            kDebug() << "UMLObject::loadFromXMI(" << m_Name << "): "
                 << "UMLStereotype " << ID2STR(stereoID)
-                << " not found" << endl;
+                << " not found, creating now." << endl;
+            setStereotype(stereo);
         }
     }
 
@@ -631,38 +671,6 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
         m_bStatic = (ownerScope == "classifier");
     }
 
-    /**** Handle XMI_FLAT_PACKAGES and old files *************************/
-    QString pkg = element.attribute( "packageid", "-1" );
-    // Some interim versions used "packageid" so test for it.
-    Uml::IDType pkgId = Uml::id_None;
-    if (pkg != "-1") {
-        pkgId = STR2ID(pkg);
-    } else {
-        pkg = element.attribute( "package", "" );
-        if (! pkg.isEmpty()) {
-            if (pkg.contains(QRegExp("\\D")))
-                // Old versions saved the package name instead of the xmi.id.
-                setPackage( pkg );
-            else
-                pkgId = STR2ID(pkg);
-        }
-    }
-    if (pkgId != Uml::id_None) {
-        UMLObject *pkgObj = umldoc->findObjectById( pkgId );
-        if (pkgObj != NULL) {
-            m_pUMLPackage = dynamic_cast<UMLPackage *>(pkgObj);
-            if (m_pUMLPackage == NULL)  // soft error
-                kdError() << "UMLObject::loadFromXMI(" << m_Name
-                << "): object of packageid "
-                << ID2STR(pkgId) << " is not a package" << endl;
-        } else {  // soft error
-            kdError() << "UMLObject::loadFromXMI(" << m_Name
-            << "): cannot resolve packageid "
-            << ID2STR(pkgId) << endl;
-        }
-    }
-    /**** End of XMI_FLAT_PACKAGES and old files handling ****************/
-
     // If the node has child nodes, check whether attributes can be
     // extracted from them.
     if (element.hasChildNodes()) {
@@ -681,11 +689,11 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
                 if (vis.isEmpty())
                     vis = elem.text();
                 if (vis == "private" || vis == "private_vis")
-                      m_Vis = Uml::Visibility::Private;
+                    m_Vis = Uml::Visibility::Private;
                 else if (vis == "protected" || vis == "protected_vis")
-                  m_Vis = Uml::Visibility::Protected;
+                    m_Vis = Uml::Visibility::Protected;
                 else if (vis == "implementation")
-                  m_Vis = Uml::Visibility::Implementation;
+                    m_Vis = Uml::Visibility::Implementation;
             } else if (Uml::tagEq(tag, "isAbstract")) {
                 QString isAbstract = elem.attribute("xmi.value", "");
                 if (isAbstract.isEmpty())
@@ -696,29 +704,8 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
                 if (ownerScope.isEmpty())
                     ownerScope = elem.text();
                 m_bStatic = (ownerScope == "classifier");
-            } else if (Uml::tagEq(tag, "stereotype")) {
-                QString stereo = elem.attribute("xmi.value", "");
-                if (stereo.isEmpty() && elem.hasChildNodes()) {
-                    /* like so:
-                     <UML:ModelElement.stereotype>
-                       <UML:Stereotype xmi.idref = '07CD'/>
-                     </UML:ModelElement.stereotype>
-                     */
-                    QDomNode stereoNode = elem.firstChild();
-                    QDomElement stereoElem = stereoNode.toElement();
-                    tag = stereoElem.tagName();
-                    if (Uml::tagEq(tag, "Stereotype")) {
-                        stereo = stereoElem.attribute("xmi.idref", "");
-                    }
-                }
-                if (! stereo.isEmpty()) {
-                    Uml::IDType stereoID = STR2ID(stereo);
-                    m_pStereotype = umldoc->findStereotypeById(stereoID);
-                    if (m_pStereotype)
-                        m_pStereotype->incrRefCount();
-                    else
-                        m_SecondaryId = stereo;  // leave it to resolveRef()
-                }
+            } else {
+                loadStereotype(elem);
             }
             node = node.nextSibling();
             if (node.isComment())
@@ -727,12 +714,12 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
         }
     }
 
-    // Operations, attributes, enum literals, stereotypes, associations,
+    // Operations, attributes, enum literals, templates, stereotypes,
     // and association role objects get added and signaled elsewhere.
     if (m_BaseType != Uml::ot_Operation && m_BaseType != Uml::ot_Attribute &&
             m_BaseType != Uml::ot_EnumLiteral && m_BaseType != Uml::ot_EntityAttribute &&
-            m_BaseType != Uml::ot_Stereotype &&
-            m_BaseType != Uml::ot_Association && m_BaseType != Uml::ot_Role) {
+            m_BaseType != Uml::ot_Template && m_BaseType != Uml::ot_Stereotype &&
+            m_BaseType != Uml::ot_Role) {
         if (m_bInPaste) {
             m_pUMLPackage = NULL;  // forget any old parent
             UMLListView *listView = UMLApp::app()->getListView();
@@ -746,10 +733,12 @@ bool UMLObject::loadFromXMI( QDomElement & element) {
                     m_pUMLPackage = static_cast<UMLPackage*>( o );
                 }
             }
-        } else if (m_pUMLPackage) {
+        }
+        if (m_pUMLPackage) {
             m_pUMLPackage->addObject(this);
         } else if (umldoc->rootFolderType(this) == Uml::N_MODELTYPES) {
-            kdDebug() << "UMLObject::loadFromXMI(" << m_Name << "): m_pUMLPackage is not set"
+            // m_pUMLPackage is not set on the root folders.
+            kDebug() << "UMLObject::loadFromXMI(" << m_Name << "): m_pUMLPackage is not set"
                 << endl;
         }
     }
